/// <reference types="react" />
import { CSSProperties } from 'react';
import { BasicConfig, BasicContainer, BasicContainerPropsInterface } from '../../render/core/Container/types';
import { ButtonType } from '@native-ads/antd/lib/button';
import './Button.less';
import { ModalConfig } from '../Modal/Modal';
export declare class NotificationConfig extends ModalConfig {
    /**
     * 提示框标题
     */
    title?: string;
    /**
     * 宽度
     */
    width?: string | number;
    /**
     * 确认按钮文字
     */
    okText?: string;
    /**
     * 确认按钮类型
     */
    okType?: ButtonType;
    /**
     * 取消按钮文字
     */
    cancelText?: string;
    /**
     * 图标 Icon 类型
     */
    iconType?: string;
    /**
     * 点击蒙层是否允许关闭
     */
    maskClosable?: boolean;
    /**
     * 设置 z-index
     */
    zIndex?: number;
    /**
     * 设置自定义内容
     */
    controls: BasicConfig[];
}
export declare class ButtonConfig extends BasicConfig {
    /**
     * 按钮的文字
     * @public
     */
    text: string;
    /**
     * 按钮的类型
     * @public
     */
    buttonType?: 'primary' | 'default' | 'alt' | 'solid' | 'link' | 'link-primary' | undefined;
    /**
     * 按钮的HTML类型 https://developer.mozilla.org/en-US/docs/Web/HTML/Element/button#attr-type
     * @public
     */
    htmlType?: 'submit' | 'button' | 'reset';
    /**
     * 设置之后, 会弹出一个确认框, 来确认是否要触发click事件
     */
    confirm?: {
        /**
         * 确认框的描述
         */
        title?: string;
        /**
         * 确认按钮文字
         */
        okText?: string;
        /**
         * 确认按钮类型
         */
        okType?: ButtonType;
        /**
         * 取消按钮文字
         */
        cancelText?: string;
    };
    /**
     * 设置之后, 会弹出一个提示框, 提示相关事件
     */
    notification?: NotificationConfig;
    /**
     * 按钮的图标类型
     * @public
     */
    icon?: string;
    /**
     * 按钮形态
     * @public
     */
    shape?: 'circle' | 'circle-outline' | undefined;
    /**
     * 按钮大小
     * @public
     */
    size?: 'small' | 'large' | undefined;
    /**
     * 加载形态
     * @public
     */
    loading?: boolean;
    /**
     * 按钮背景透明
     * @public
     */
    ghost?: boolean;
    /**
     * 是否禁用
     * @public
     */
    disabled?: boolean;
    /**
     * unknown
     */
    prefixCls?: string;
    /**
     * CSS class
     */
    className?: string;
    /**
     * 内联CSS属性
     */
    style?: CSSProperties;
    /**
     * 跳转链接
     */
    href?: string;
}
export declare class ButtonPropsInterface extends BasicContainerPropsInterface {
    info: ButtonConfig;
}
declare class AbstractButton extends BasicContainer<ButtonPropsInterface, {}> {
    constructor(props: ButtonPropsInterface);
    render(): {} | null | undefined;
    private mapButtonOptions(info);
    private mapModalOptions(info);
}
export default AbstractButton;
