/// <reference types="react" />
import { CSSProperties } from 'react';
import { BasicConfig, BasicContainer, BasicContainerPropsInterface } from '../../render/core/Container/types';
import { CascaderOptionType } from '@native-ads/antd/lib/cascader';
export declare class CascaderConfig extends BasicConfig {
    /**
     * Checkbox的数据模型Key
     */
    name: string;
    /**
     * 是否支持清除
     */
    allowClear?: boolean;
    /**
     * CSS class
     */
    className?: string;
    /**
     * 内联CSS属性
     */
    style?: CSSProperties;
    /**
     * 禁用
     */
    disabled?: boolean;
    /**
     * 次级菜单的展开方式，可选 'click' 和 'hover'
     */
    expandTrigger?: 'click' | 'hover';
    /**
     * 可选项数据源
     */
    options: CascaderOptionType[];
    /**
     * 自定义浮层类名
     */
    popupClassName?: string;
    /**
     *  浮层预设位置：`bottomLeft` `bottomRight` `topLeft` `topRight`
     */
    popupPlacement?: string;
    /**
     * 输入框占位文本
     */
    placeholder?: string;
    /**
     * 输入框大小，可选 `large` `default` `small`
     */
    size?: string;
    /**
     * 是否加载中
     */
    loading?: boolean;
    /**
     * 在选择框中显示搜索框
     */
    showSearch?: boolean;
    /**
     * options属性映射
     */
    optionsMapping?: CascaderOptionType;
    /**
     * 组件加载初始值
     */
    defaultValue: (string | number)[];
    /**
     * 选择即改变
     */
    changeOnSelect?: boolean;
}
export declare class CascaderPropsInterface extends BasicContainerPropsInterface {
    info: CascaderConfig;
    /**
     * 当标签页展开，用于自定义扩展
     */
    onPopupVisibleChange?: (popupVisible: boolean) => Object;
    /**
     * 当数据更新
     */
    onChange?: (value: string[], selectedOptions?: CascaderOptionType[]) => Object;
}
export declare class AbstractCascader extends BasicContainer<CascaderPropsInterface, {}> {
    constructor(props: CascaderPropsInterface);
    componentWillMount(): void;
    private mapCascaderOptions(info);
    private handleChange(value, selectedOptions?);
    private mapOptions(options, optionsMapping);
    render(): JSX.Element;
}
