/// <reference types="react" />
import * as React from 'react';
import { CSSProperties } from 'react';
import { BasicConfig, BasicContainer, BasicContainerPropsInterface } from '../../render/core/Container/types';
export declare class CheckboxConfig extends BasicConfig {
    /**
     * Checkbox的数据模型Key
     */
    name: string;
    /**
     * 是否禁用
     * @public
     * @default false
     */
    disabled?: boolean;
    /**
     * 文字
     * @public
     * @default ''
     */
    text?: string;
    /**
     * 多组多选框
     */
    groups?: CheckBoxGroupItem[];
    /**
     * 添加全选框
     */
    groupSelectAll?: boolean;
    /**
     * 添加反选框
     */
    groupUnSelectAll?: boolean;
    /**
     * 初始是否选中
     */
    defaultValue?: boolean | string[];
    /**
     * unknown
     */
    prefixCls?: string;
    /**
     * CSS class
     */
    className?: string;
    /**
     * 内联CSS属性
     */
    style?: CSSProperties;
}
export declare class CheckBoxGroupItem {
    /**
     * 多选的Key
     */
    key: string;
    /**
     * 多选框文字
     */
    text: string;
    /**
     * 提示文字
     */
    tip?: string;
}
export declare class CheckboxPropsInterface extends BasicContainerPropsInterface {
    info: CheckboxConfig;
    /**
     * 当鼠标移入的回调
     */
    onMouseEnter?: (event: React.MouseEvent<HTMLInputElement>) => Object;
    /**
     * 当鼠标移开的回调
     */
    onMouseLeave?: (event: React.MouseEvent<HTMLInputElement>) => Object;
    /**
     * 当数据更新的回调
     */
    onChange?: (event: React.MouseEvent<HTMLInputElement>) => Object;
}
export default class AbstractCheckbox extends BasicContainer<CheckboxPropsInterface, {}> {
    private SELECT_ALL;
    constructor(props: CheckboxPropsInterface);
    private initCheckbox(info);
    componentWillMount(): void;
    componentWillReceiveProps(nextProps: CheckboxPropsInterface): void;
    private handleChange(event);
    private handleGroupChange(key);
    private selectAll(event);
    private mapOptions(info);
    render(): JSX.Element;
}
