/// <reference types="react" />
import { CSSProperties } from 'react';
import { BasicConfig, BasicContainer, BasicContainerPropsInterface } from '../../render/core/Container/types';
export declare class DatePickerCommonConfig extends BasicConfig {
    /**
     * 数据模型Key
     */
    name: string;
    /**
     * 是否禁用
     * @public
     * @default false
     */
    disabled?: boolean;
    /**
     * 是否显示清除按钮
     */
    allowClear?: boolean;
    /**
     * CSS样式
     */
    style?: CSSProperties;
    /**
     * CSS class
     */
    className?: string;
}
export declare class DatePickerConfig extends DatePickerCommonConfig {
    /**
     * 开始时间
     * @public
     * @default null
     */
    startTime?: string | number;
    /**
     * 结束时间
     * @public
     * @default null
     */
    endTime?: string | number;
    /**
     * 展示的日期格式，配置参考 moment.js
     */
    format?: string;
    /**
     * 是否展示“今天”按钮
     */
    showToday?: boolean;
    /**
     * 选择的日期可以被清除
     */
    clearAble?: boolean;
    /**
     * 增加时间选择功能
     */
    showTime?: boolean;
    /**
     * 输入框高度
     */
    size: 'large' | 'small';
    /**
     * 初始默认值
     */
    defaultValue?: string;
    /**
     * 输入框提示文字
     * @public
     * @default ''
     */
    placeholder?: string;
    /**
     * 自定义输入框模板
     */
    dateInputTemplate?: string;
}
export declare class DatePickerPropsInterface extends BasicContainerPropsInterface {
    info: DatePickerConfig;
}
export default class AbstractDatepicker extends BasicContainer<DatePickerPropsInterface, {}> {
    constructor(props: DatePickerPropsInterface);
    componentWillMount(): void;
    render(): JSX.Element;
    private mapDatePickerOptions(info);
    private disabledDate(info);
    private handleChange(date, dateString);
}
