/// <reference types="react" />
import * as React from 'react';
import { BasicConfig, BasicContainer, BasicContainerPropsInterface } from '../../render/core/Container/types';
export declare class ModalConfig extends BasicConfig {
    /**
     * Modal的数据模型Key
     */
    name: string;
    /**
     * 确定按钮 loading
     */
    confirmLoading?: boolean;
    /**
     * 标题
     */
    title?: string;
    /**
     * 是否显示右上角的关闭按钮
     */
    closable?: boolean;
    /**
     * 宽度
     */
    width?: number | string;
    /**
     * 底部内容
     */
    footer?: BasicConfig[];
    /**
     * 确认按钮文字
     */
    okText?: string;
    /**
     * 点击确认作为表单提交操作
     */
    okToSubmit: boolean;
    /**
     * 关闭时销毁 Modal 里的子元素
     */
    destroyOnClose?: boolean;
    /**
     * 确认按钮类型
     */
    okType?: 'primary' | 'default' | 'alt' | 'solid' | 'link' | 'link-primary' | undefined;
    /**
     * 取消文字
     */
    cancelText?: string;
    /**
     * 点击蒙层是否允许关闭
     */
    maskClosable?: boolean;
    /**
     * 对话框外层容器的类名
     */
    wrapClassName?: string;
    /**
     * 设置 Modal 的 z-index
     */
    zIndex?: number;
    /**
     * 字级内容
     */
    children: BasicConfig[];
}
export declare class ModalPropsInterface extends BasicContainerPropsInterface {
    info: ModalConfig;
    /**
     * 触发表单提交操作
     */
    $formSubmit?: () => void;
}
export declare class AbstractModal extends BasicContainer<ModalPropsInterface, {}> {
    constructor(props: ModalPropsInterface);
    handleOk(event: React.MouseEvent<HTMLDivElement>): void;
    handleCancel(event: React.MouseEvent<HTMLDivElement>): void;
    private mapModalOptions(info);
    render(): JSX.Element;
}
