/// <reference types="react" />
import { CSSProperties } from 'react';
import { BasicConfig, BasicContainer, BasicContainerPropsInterface } from '../../render/core/Container/types';
export declare class RadioOptionItem {
    /**
     * 多组单选按钮的文字
     */
    label: string;
    /**
     * 多组单选按钮的值
     */
    value: string;
    /**
     * 禁用选项
     */
    disabled?: boolean;
}
export declare class RadioConfig extends BasicConfig {
    /**
     * Radil的数据模型Key
     */
    name: string;
    /**
     * 初始的值
     */
    defaultValue?: string;
    /**
     * 模式
     */
    mode: 'radio' | 'button' | 'menu';
    /**
     * 是否禁用
     * @public
     * @default false
     */
    disabled: boolean;
    /**
     * unknown
     */
    prefixCls?: string;
    /**
     * 单个选择框的文字
     * @public
     * @default ''
     */
    text?: string;
    /**
     * 多组单选按钮.
     */
    options?: RadioOptionItem[];
    /**
     * CSS class
     */
    className?: string;
    /**
     * RadioGroup中, 按钮的大小
     */
    size: 'large' | 'default' | 'small';
    /**
     * 内联CSS属性
     */
    style?: CSSProperties;
}
export declare class RadioPropsInterface extends BasicContainerPropsInterface {
    info: RadioConfig;
}
export declare class AbstractRadio extends BasicContainer<RadioPropsInterface, {}> {
    constructor(props: RadioPropsInterface);
    componentWillMount(): void;
    componentWillReceiveProps(nextProps: RadioPropsInterface): void;
    private mapOptions(info);
    private mapRadioGroupOptions(info);
    private handleChange(event);
    private handleRadioGroupChange(event);
    render(): JSX.Element;
}
