/// <reference types="react" />
import { CSSProperties } from 'react';
import { BasicConfig, BasicContainer, BasicContainerPropsInterface } from '../../render/core/Container/types';
export declare class SliderConfig extends BasicConfig {
    /**
     * Slider组件的数据模型Key
     */
    name: string;
    /**
     * 初始化默认值
     */
    defaultValue: number | [number, number];
    /**
     * 是否禁用 默认false
     */
    disabled?: boolean;
    /**
     * 是否只能拖拽到刻度上, 默认false
     */
    dots?: boolean;
    /**
     * marks 不为空对象时有效，值为 true 时表示值为包含关系，false 表示并列
     */
    included?: boolean;
    /**
     * 刻度标记，key 的类型必须为 number 且取值在闭区间 min, max 内，每个标签可以单独设置样式
     */
    marks?: any;
    /**
     * 最大值
     */
    max: number;
    /**
     * 最小值
     */
    min: number;
    /**
     * 双滑块模式
     */
    range?: boolean;
    /**
     * Slider 会把当前值传给 tipFormatter，并在 Tooltip 中显示 tipFormatter 的返回值
     */
    step: number | null;
    /**
     * 使用ExpressString来格式化显示的值
     */
    tipFormat: string;
    /**
     * 设置当前取值.
     */
    value: number | [number, number] | undefined;
    /**
     * 设置为垂直方向
     */
    vertical?: boolean;
    /**
     * 内联CSS属性
     */
    style?: CSSProperties;
}
export declare class SliderPropsInterface extends BasicContainerPropsInterface {
    info: SliderConfig;
}
export declare class SliderStateInterface {
}
export declare class AbstractSlider extends BasicContainer<SliderPropsInterface, SliderStateInterface> {
    constructor(props: SliderPropsInterface);
    componentWillMount(): void;
    private mapSliderProps(info);
    /**
     * 对于onChange事件，要进行直接同步值到数据模型中
     * @param {number} value
     * @return {void}
     */
    private handleChange(value);
    render(): JSX.Element;
}
