/// <reference types="react" />
import { CSSProperties } from 'react';
import { BasicConfig, BasicContainer, BasicContainerPropsInterface } from '../../render/core/Container/types';
import { TreeNodeConfig } from '../Tree/Tree';
export declare class TreeSelectConfig extends BasicConfig {
    /**
     * 同步的数据模型Key
     */
    name: string;
    /**
     * 子节点配置
     */
    options: TreeNodeConfig[];
    /**
     * 显示清除按钮
     */
    allowClear?: boolean;
    /**
     * 默认值
     */
    defaultValue?: string | string[];
    /**
     * 是否禁用
     */
    disabled?: boolean;
    /**
     * 下拉菜单和选择器同宽
     */
    dropdownMatchSelectWidth?: boolean;
    /**
     * 下拉菜单的样式
     */
    dropdownStyle?: CSSProperties;
    /**
     * 输入项过滤
     */
    filterTreeNode?: boolean | string;
    /**
     * 是否把每个选项的 label 包装到 value 中
     */
    labelInValue?: boolean;
    /**
     * checkable 状态下节点选择完全受控（父子节点选中状态不再关联），会使得 labelInValue 强制为 true
     */
    treeCheckStrictly?: boolean;
    /**
     * 是否多选
     */
    multiple?: boolean;
    /**
     * 选择框默认文字
     */
    placeholder?: string;
    /**
     * 搜索框默认文字
     */
    searchPlaceholder?: string;
    /**
     * 定义选中项回填的方式
     *
     * SHOW_ALL: 显示所有选中节点(包括父节点)
     * SHOW_PARENT: 只显示父节点(当父节点下所有子节点都选中时).
     * SHOW_CHILD: 只显示子节点. (默认)
     */
    showCheckedStrategy?: 'SHOW_ALL' | 'SHOW_PARENT' | 'SHOW_CHILD';
    /**
     * 是否显示搜索框
     */
    showSearch?: boolean;
    /**
     * 选择框大小
     */
    size?: 'default' | 'large' | 'small';
    /**
     * checkbox多选
     */
    treeCheckable?: boolean;
    /**
     * 默认展开所有的节点
     */
    treeDefaultExpandAll?: boolean;
    /**
     * 默认展开的节点
     */
    treeDefaultExpandedKeys?: string[];
}
export declare class TreeSelectPropsInterface extends BasicContainerPropsInterface {
    info: TreeSelectConfig;
}
export declare class AbstractTreeSelect extends BasicContainer<TreeSelectPropsInterface, {}> {
    constructor(props: TreeSelectPropsInterface);
    componentDidMount(): void;
    private handleTreeChange(value);
    private parseFilterTreeNode(inputValue, treeNode);
    private renderTreeNode(options);
    render(): JSX.Element;
}
