/// <reference types="react" />
import { actionCreators } from './action';
import * as PropTypes from 'prop-types';
import * as React from 'react';
import { CSSProperties } from 'react';
import * as moment from 'moment';
import { ContainerConfig } from '../../../components/Container/Container';
import { TriggerEventItem } from '../Trigger/Trigger';
import { DataCustomer } from '../DataCustomer/Controller';
import { RCREOptions } from '../Page';
export declare type rawJSONType = string | number | null | boolean | Object;
export declare type originJSONType = rawJSONType | rawJSONType[];
export declare type defaultData = {
    [s: string]: originJSONType;
};
export declare class BasicConfig {
    /**
     * 组件类型
     */
    type: string;
    /**
     * 是否隐藏
     */
    hidden?: boolean;
    /**
     * 字级数据Key
     */
    name?: string;
    /**
     * 是否显示
     */
    show?: boolean;
    /**
     * 父级属性映射
     */
    parentMapping?: Object;
    /**
     * 子级元素
     */
    children?: BasicConfig[];
    /**
     * CSS class
     */
    className?: string;
    /**
     * 内联CSS属性
     */
    style?: CSSProperties;
    /**
     * 事件触发
     */
    trigger?: TriggerEventItem[];
    /**
     * 是否作为表单输入元素
     */
    formItem?: boolean;
    [s: string]: any;
}
export declare class BasicContainerPropsInterface {
    info: BasicConfig;
    /**
     * 当前Container的数据模型对象
     */
    $data?: Object;
    /**
     * 父级Container组件的数据模型对象
     */
    $parent?: Object;
    /**
     * RCRE 渲染配置
     */
    options?: RCREOptions;
    /**
     * Trigger组件的数据模型对象
     */
    $trigger?: Object;
    /**
     * 通过表格组件, 渲染之后, 获取到的每一行的数据
     */
    $item?: Object;
    /**
     * 通过表格组件, 渲染之后, 获取到的第几行
     */
    $index?: number;
    /**
     * 表单状态变量
     */
    $form?: {
        [s: string]: any;
    };
    /**
     * React组件Key
     */
    key?: string | number;
    /**
     * 底层组件设置数据模型值使用
     */
    $setData?: (name: string, value: any) => void;
    /**
     * 底层组件清除某个字段的数据
     */
    $deleteData?: (name: string) => void;
    /**
     * 底层组件设置多组数据模型值
     */
    $setMultiData?: (items: {
        name: string;
        value: any;
    }[]) => void;
    /**
     * Trigger注入的通用事件处理函数, 所有事件处理都走这里
     */
    eventHandle?: (eventName: string, args: Object) => void;
    /**
     * 来自Container的数据消耗者实例
     */
    dataCustomer?: DataCustomer;
    /**
     * 父级的数据模型Key
     */
    model?: string;
}
export declare class ContainerProps extends BasicContainerPropsInterface {
    info: ContainerConfig;
    $data: {};
    $parent: {};
    options: RCREOptions;
    /**
     * 写入数据到数据模型
     */
    setData: typeof actionCreators.setData;
    /**
     * 批量写入一组数据
     */
    setDataList: typeof actionCreators.setDataList;
    /**
     * 批量写入多组不同key的数据
     */
    setMultiData: typeof actionCreators.setMultiData;
    /**
     * 清空当前数据模型
     */
    clearData: typeof actionCreators.clearData;
    /**
     * 删除数据模型某一个字段
     */
    deleteData: typeof actionCreators.deleteData;
    /**
     * 异步加载数据中
     */
    asyncLoadDataProgress: typeof actionCreators.asyncLoadDataProgress;
    /**
     * 异步加载数据成功
     */
    asyncLoadDataSuccess: typeof actionCreators.asyncLoadDataSuccess;
    /**
     * 异步加载数据失败
     */
    asyncLoadDataFail: typeof actionCreators.asyncLoadDataFail;
    /**
     * 同步加载数据成功
     */
    syncLoadDataSuccess: typeof actionCreators.syncLoadDataSuccess;
    /**
     * 同步加载数据失败
     */
    syncLoadDataFail: typeof actionCreators.syncLoadDataFail;
}
export declare const BasicContextTypes: {
    driver: PropTypes.Requireable<any>;
    form: PropTypes.Requireable<any>;
    abstractContainer: PropTypes.Requireable<any>;
    $store: PropTypes.Requireable<any>;
    $global: PropTypes.Requireable<any>;
    $location: PropTypes.Requireable<any>;
    $query: PropTypes.Requireable<any>;
    debug: PropTypes.Requireable<any>;
    lang: PropTypes.Requireable<any>;
};
export declare type runTimeType = {
    $data?: Object;
    $query?: Object;
    $global?: any;
    $item?: Object;
    $trigger?: {
        $SELF_PASS_CUSTOMER?: Object;
    };
    $index?: number;
    $now?: moment.Moment;
    $moment: typeof moment;
    $args?: Object;
    $output?: Object;
    $iterator?: Object;
    $parent?: Object;
    $form?: Object;
    $prev?: any;
};
/**
 * 获取ExpressionString 嵌入的上下文
 * @param {BasicContainerPropsInterface} props
 * @param context
 * @return {runTimeType}
 */
export declare function getRuntimeContext(props: BasicContainerPropsInterface, context: any): runTimeType;
/**
 * 所有子级组件的基类
 */
export declare abstract class BasicContainer<T extends BasicContainerPropsInterface, P> extends React.Component<T, P> {
    static contextTypes: {
        driver: PropTypes.Requireable<any>;
        form: PropTypes.Requireable<any>;
        abstractContainer: PropTypes.Requireable<any>;
        $store: PropTypes.Requireable<any>;
        $global: PropTypes.Requireable<any>;
        $location: PropTypes.Requireable<any>;
        $query: PropTypes.Requireable<any>;
        debug: PropTypes.Requireable<any>;
        lang: PropTypes.Requireable<any>;
    };
    private depESProps;
    private computedCache;
    private cacheBlackList;
    constructor(props: T);
    componentWillUnmount(): void;
    shouldComponentUpdate(nextProps: T, nextState: P): boolean;
    /**
     * 获取当前组件的ExpressionString 内嵌变量
     *
     * @param {T} props React组件的Props
     * @param context React组件Context
     * @returns {runTimeType}
     */
    protected getRuntimeContext(props?: T, context?: any): runTimeType;
    /**
     * 调用此函数来解析JSON配置中的ExpressionString指令
     *
     * @param {InfoType} info 组件的JSON配置
     * @param {T} props React组件的Props
     * @param {string[]} blackList 属性黑名单，指定之后不处理ExpressionString
     * @param {boolean} isDeep 是否深度遍历JSON配置
     * @param {string[]} whiteList 属性白名单，如果指定之后，只处理白名单里面的属性值
     * @returns {InfoType}
     */
    protected getPropsInfo<InfoType>(info: InfoType, props?: T, blackList?: string[], isDeep?: boolean, whiteList?: string[]): InfoType;
    /**
     * 错误处理函数，用于返回一些可预见的错误。
     *
     * @param {string} msg
     * @param extendElement
     * @returns {React.DetailedReactHTMLElement<React.InputHTMLAttributes<HTMLInputElement>, HTMLInputElement>}
     */
    protected errorReport(msg: string, extendElement: any): React.ComponentElement<{}, React.Component<{}, React.ComponentState>>;
    /**
     * 渲染组件，用于支持hidden属性，可让组件隐藏
     *
     * @param {BasicConfig} info 组件的JSON配置
     * @param {React.ReactElement} children 输出的React组件
     * @returns {any}
     */
    protected renderChildren(info: BasicConfig, children: React.ReactNode): {} | null | undefined;
    /**
     * 自动获取用于传递给子级组件的props属性
     *
     * @param {BasicConfig} info
     * @param {Object} childProps
     * @returns {BasicContainerPropsInterface}
     */
    protected getChildProps(info: BasicConfig, childProps: Object): {
        constructor: Function;
        toString(): string;
        toLocaleString(): string;
        valueOf(): Object;
        hasOwnProperty(v: string): boolean;
        hasOwnProperty(v: PropertyKey): boolean;
        isPrototypeOf(v: Object): boolean;
        propertyIsEnumerable(v: string): boolean;
        propertyIsEnumerable(v: PropertyKey): boolean;
        info: BasicConfig;
        $data: T["$data"];
        $parent: T["$parent"];
        model: T["model"];
        dataCustomer: T["dataCustomer"];
        $setData: T["$setData"];
        eventHandle: T["eventHandle"];
        $index: T["$index"];
        $item: T["$item"];
    };
    /**
     * 事件注册器，注册一个事件，可以使用trigger属性来调用
     *
     * @param {string} eventName 事件名称
     * @param {{[p: string]: any}} args 额外的参数，会注入到param属性中的$args对象
     * @param {boolean} mute
     */
    protected commonEventHandler(eventName: string, args: {
        [s: string]: any;
    }, mute?: boolean): void;
    /**
     * 替换this.props.$data.get(name)这样的用法，来支持name属性中带有.分割的情况
     *
     * @param {string} nameStr 组件的name属性
     * @returns {any}
     */
    protected getValueFromDataStore(nameStr: string, $data?: Object): any;
    /**
     * 组件是否处于container组件内部
     * @returns {boolean}
     */
    protected isUnderContainerEnv(): boolean;
    /**
     *
     * @returns {boolean}
     */
    protected isDebugMode(): boolean;
    /**
     * 当使用扩展组件来使用的时候才提供参数，正常不提供参数
     * @param {any[]} args
     * @param {(event: any) => Object} callback
     * @returns {Object}
     */
    protected getExternalCallbackArgs(args: any[], callback?: (event: any) => Object): Object;
}
