import { runTimeType } from '../core/Container/types';
export declare type TokenItem = {
    start: number;
    end: number;
    str?: string;
    token?: string;
    prev?: boolean;
};
/**
 * 解析ExpressString, 取出#ES{}内部的代码传入到runTime
 *
 * @param {string} str
 * @param {runTimeType} context
 * @returns {any}
 */
export declare function parseExpressString(str: string, context: runTimeType): any;
/**
 * 处理ExpressionString, 得到Token
 *
 * @param {string} str
 * @returns {TokenItem[]}
 */
export declare function parseExpressionToken(str: string): TokenItem[];
/**
 * 执行token, 计算出运行结果
 *
 * @param {TokenItem[]} tokens
 * @param {Object} context
 * @returns {TokenItem}
 */
export declare function execExpressionTokens(tokens: TokenItem[], context: Object): any;
/**
 * 是否包含函数调用
 *
 * @param {string} code
 * @returns {boolean}
 */
export declare function isFuncCall(code: string): boolean;
/**
 * 取出函数调用的参数值
 *
 * @param {string} code
 * @returns {(string | number | boolean)[]}
 */
export declare function getParamsFromFuncCall(code: string): (string | number | boolean | null | undefined)[];
/**
 * 判断一个字符串是否是ExpressionString
 * @param {string} str
 * @return {boolean}
 */
export declare function isExpressionString(str: string): boolean;
